@@ -11,4 +11,4 @@ class ArgumentException(Exception):
 
 
 class GraphTargetMissingException(WorkflowException):
-    """When a $graph is encountered and there is no target and no main/#main."""
+    """When a ``$graph`` is encountered and there is no target and no ``main``/``#main``."""
