@@ -4,6 +4,7 @@ import functools
 import logging
 import threading
 from typing import (
+    TYPE_CHECKING,
     Dict,
     List,
     MutableMapping,
@@ -18,7 +19,6 @@ from typing import (
 from cwl_utils import expression
 from schema_salad.sourceline import SourceLine
 from schema_salad.utils import json_dumps
-from typing_extensions import TYPE_CHECKING
 
 from .builder import content_limit_respected_read
 from .checker import can_assign_src_to_sink
@@ -56,7 +56,7 @@ class WorkflowJobStep:
         self.tool = step.tool
         self.id = step.id
         self.submitted = False
-        self.iterable = None  # type: Optional[JobsGeneratorType]
+        self.iterable: Optional[JobsGeneratorType] = None
         self.completed = False
         self.name = uniquename("step %s" % shortname(self.id))
         self.prov_obj = step.prov_obj
@@ -92,11 +92,10 @@ class ReceiveScatterOutput:
         self.processStatus = "success"
         self.total = total
         self.output_callback = output_callback
-        self.steps = []  # type: List[Optional[JobsGeneratorType]]
+        self.steps: List[Optional[JobsGeneratorType]] = []
 
-    def receive_scatter_output(
-        self, index: int, jobout: CWLObjectType, processStatus: str
-    ) -> None:
+    def receive_scatter_output(self, index: int, jobout: CWLObjectType, processStatus: str) -> None:
+        """Record the results of a scatter operation."""
         for key, val in jobout.items():
             self.dest[key][index] = val
 
@@ -138,9 +137,10 @@ def parallel_steps(
     while rc.completed < rc.total:
         made_progress = False
         for index, step in enumerate(steps):
-            if getdefault(
-                runtimeContext.on_error, "stop"
-            ) == "stop" and rc.processStatus not in ("success", "skipped"):
+            if getdefault(runtimeContext.on_error, "stop") == "stop" and rc.processStatus not in (
+                "success",
+                "skipped",
+            ):
                 break
             if step is None:
                 continue
@@ -174,19 +174,17 @@ def nested_crossproduct_scatter(
 ) -> JobsGeneratorType:
     scatter_key = scatter_keys[0]
     jobl = len(cast(Sized, joborder[scatter_key]))
-    output = {}  # type: ScatterDestinationsType
+    output: ScatterDestinationsType = {}
     for i in process.tool["outputs"]:
         output[i["id"]] = [None] * jobl
 
     rc = ReceiveScatterOutput(output_callback, output, jobl)
 
-    steps = []  # type: List[Optional[JobsGeneratorType]]
+    steps: List[Optional[JobsGeneratorType]] = []
     for index in range(0, jobl):
-        sjob = copy.copy(joborder)  # type: Optional[CWLObjectType]
+        sjob: Optional[CWLObjectType] = copy.copy(joborder)
         assert sjob is not None  # nosec
-        sjob[scatter_key] = cast(
-            MutableMapping[int, CWLObjectType], joborder[scatter_key]
-        )[index]
+        sjob[scatter_key] = cast(MutableMapping[int, CWLObjectType], joborder[scatter_key])[index]
 
         if len(scatter_keys) == 1:
             if runtimeContext.postScatterEval is not None:
@@ -212,9 +210,8 @@ def nested_crossproduct_scatter(
     return parallel_steps(steps, rc, runtimeContext)
 
 
-def crossproduct_size(
-    joborder: CWLObjectType, scatter_keys: MutableSequence[str]
-) -> int:
+def crossproduct_size(joborder: CWLObjectType, scatter_keys: MutableSequence[str]) -> int:
+    """Compute the size of a cross product."""
     scatter_key = scatter_keys[0]
     if len(scatter_keys) == 1:
         ssum = len(cast(Sized, joborder[scatter_key]))
@@ -232,7 +229,7 @@ def flat_crossproduct_scatter(
     output_callback: ScatterOutputCallbackType,
     runtimeContext: RuntimeContext,
 ) -> JobsGeneratorType:
-    output = {}  # type: ScatterDestinationsType
+    output: ScatterDestinationsType = {}
     for i in process.tool["outputs"]:
         output[i["id"]] = [None] * crossproduct_size(joborder, scatter_keys)
     callback = ReceiveScatterOutput(output_callback, output, 0)
@@ -250,18 +247,16 @@ def _flat_crossproduct_scatter(
     callback: ReceiveScatterOutput,
     startindex: int,
     runtimeContext: RuntimeContext,
-) -> Tuple[List[Optional[JobsGeneratorType]], int,]:
+) -> Tuple[List[Optional[JobsGeneratorType]], int]:
     """Inner loop."""
     scatter_key = scatter_keys[0]
     jobl = len(cast(Sized, joborder[scatter_key]))
-    steps = []  # type: List[Optional[JobsGeneratorType]]
+    steps: List[Optional[JobsGeneratorType]] = []
     put = startindex
     for index in range(0, jobl):
-        sjob = copy.copy(joborder)  # type: Optional[CWLObjectType]
+        sjob: Optional[CWLObjectType] = copy.copy(joborder)
         assert sjob is not None  # nosec
-        sjob[scatter_key] = cast(
-            MutableMapping[int, CWLObjectType], joborder[scatter_key]
-        )[index]
+        sjob[scatter_key] = cast(MutableMapping[int, CWLObjectType], joborder[scatter_key])[index]
 
         if len(scatter_keys) == 1:
             if runtimeContext.postScatterEval is not None:
@@ -290,27 +285,26 @@ def dotproduct_scatter(
     output_callback: ScatterOutputCallbackType,
     runtimeContext: RuntimeContext,
 ) -> JobsGeneratorType:
-    jobl = None  # type: Optional[int]
+    jobl: Optional[int] = None
     for key in scatter_keys:
         if jobl is None:
             jobl = len(cast(Sized, joborder[key]))
         elif jobl != len(cast(Sized, joborder[key])):
             raise WorkflowException(
-                "Length of input arrays must be equal when performing "
-                "dotproduct scatter."
+                "Length of input arrays must be equal when performing " "dotproduct scatter."
             )
     if jobl is None:
         raise Exception("Impossible codepath")
 
-    output = {}  # type: ScatterDestinationsType
+    output: ScatterDestinationsType = {}
     for i in process.tool["outputs"]:
         output[i["id"]] = [None] * jobl
 
     rc = ReceiveScatterOutput(output_callback, output, jobl)
 
-    steps = []  # type: List[Optional[JobsGeneratorType]]
+    steps: List[Optional[JobsGeneratorType]] = []
     for index in range(0, jobl):
-        sjobo = copy.copy(joborder)  # type: Optional[CWLObjectType]
+        sjobo: Optional[CWLObjectType] = copy.copy(joborder)
         assert sjobo is not None  # nosec
         for key in scatter_keys:
             sjobo[key] = cast(MutableMapping[int, CWLObjectType], joborder[key])[index]
@@ -386,7 +380,7 @@ def object_from_state(
     sourceField: str,
     incomplete: bool = False,
 ) -> Optional[CWLObjectType]:
-    inputobj = {}  # type: CWLObjectType
+    inputobj: CWLObjectType = {}
     for inp in params:
         iid = original_id = cast(str, inp["id"])
         if frag_only:
@@ -479,17 +473,17 @@ class WorkflowJob:
     def __init__(self, workflow: "Workflow", runtimeContext: RuntimeContext) -> None:
         """Initialize this WorkflowJob."""
         self.workflow = workflow
-        self.prov_obj = None  # type: Optional[ProvenanceProfile]
-        self.parent_wf = None  # type: Optional[ProvenanceProfile]
+        self.prov_obj: Optional[ProvenanceProfile] = None
+        self.parent_wf: Optional[ProvenanceProfile] = None
         self.tool = workflow.tool
         if runtimeContext.research_obj is not None:
             self.prov_obj = workflow.provenance_object
             self.parent_wf = workflow.parent_wf
         self.steps = [WorkflowJobStep(s) for s in workflow.steps]
-        self.state = {}  # type: Dict[str, Optional[WorkflowStateItem]]
+        self.state: Dict[str, Optional[WorkflowStateItem]] = {}
         self.processStatus = ""
         self.did_callback = False
-        self.made_progress = None  # type: Optional[bool]
+        self.made_progress: Optional[bool] = None
         self.outdir = runtimeContext.get_outdir()
 
         self.name = uniquename(
@@ -508,12 +502,11 @@ class WorkflowJob:
         )
 
     def do_output_callback(self, final_output_callback: OutputCallbackType) -> None:
-
         supportsMultipleInput = bool(
             self.workflow.get_requirement("MultipleInputFeatureRequirement")[0]
         )
 
-        wo = None  # type: Optional[CWLObjectType]
+        wo: Optional[CWLObjectType] = None
         try:
             wo = object_from_state(
                 self.state,
@@ -524,16 +517,14 @@ class WorkflowJob:
                 incomplete=True,
             )
         except WorkflowException as err:
-            _logger.error(
-                "[%s] Cannot collect workflow output: %s", self.name, str(err)
-            )
+            _logger.error("[%s] Cannot collect workflow output: %s", self.name, str(err))
             self.processStatus = "permanentFail"
         if (
             self.prov_obj
             and self.parent_wf
             and self.prov_obj.workflow_run_uri != self.parent_wf.workflow_run_uri
         ):
-            process_run_id = None  # type: Optional[str]
+            process_run_id: Optional[str] = None
             self.prov_obj.generate_output_prov(wo or {}, process_run_id, self.name)
             self.prov_obj.document.wasEndedBy(
                 self.prov_obj.workflow_run_uri,
@@ -543,9 +534,7 @@ class WorkflowJob:
             )
             prov_ids = self.prov_obj.finalize_prov_profile(self.name)
             # Tell parent to associate our provenance files with our wf run
-            self.parent_wf.activity_has_provenance(
-                self.prov_obj.workflow_run_uri, prov_ids
-            )
+            self.parent_wf.activity_has_provenance(self.prov_obj.workflow_run_uri, prov_ids)
 
         _logger.info("[%s] completed %s", self.name, self.processStatus)
         if _logger.isEnabledFor(logging.DEBUG):
@@ -563,21 +552,16 @@ class WorkflowJob:
         jobout: CWLObjectType,
         processStatus: str,
     ) -> None:
-
         for i in outputparms:
             if "id" in i:
                 iid = cast(str, i["id"])
                 if iid in jobout:
                     self.state[iid] = WorkflowStateItem(i, jobout[iid], processStatus)
                 else:
-                    _logger.error(
-                        "[%s] Output is missing expected field %s", step.name, iid
-                    )
+                    _logger.error("[%s] Output is missing expected field %s", step.name, iid)
                     processStatus = "permanentFail"
         if _logger.isEnabledFor(logging.DEBUG):
-            _logger.debug(
-                "[%s] produced output %s", step.name, json_dumps(jobout, indent=4)
-            )
+            _logger.debug("[%s] produced output %s", step.name, json_dumps(jobout, indent=4))
 
         if processStatus not in ("success", "skipped"):
             if self.processStatus != "permanentFail":
@@ -632,13 +616,9 @@ class WorkflowJob:
                 self.receive_output, step, outputparms, final_output_callback
             )
 
-            valueFrom = {
-                i["id"]: i["valueFrom"] for i in step.tool["inputs"] if "valueFrom" in i
-            }
+            valueFrom = {i["id"]: i["valueFrom"] for i in step.tool["inputs"] if "valueFrom" in i}
 
-            loadContents = {
-                i["id"] for i in step.tool["inputs"] if i.get("loadContents")
-            }
+            loadContents = {i["id"] for i in step.tool["inputs"] if i.get("loadContents")}
 
             if len(valueFrom) > 0 and not bool(
                 self.workflow.get_requirement("StepInputExpressionRequirement")[0]
@@ -658,13 +638,9 @@ class WorkflowJob:
                             with fs_access.open(cast(str, val["location"]), "rb") as f:
                                 val["contents"] = content_limit_respected_read(f)
 
-                def valueFromFunc(
-                    k: str, v: Optional[CWLOutputType]
-                ) -> Optional[CWLOutputType]:
+                def valueFromFunc(k: str, v: Optional[CWLOutputType]) -> Optional[CWLOutputType]:
                     if k in valueFrom:
-                        adjustDirObjs(
-                            v, functools.partial(get_listing, fs_access, recursive=True)
-                        )
+                        adjustDirObjs(v, functools.partial(get_listing, fs_access, recursive=True))
 
                         return expression.do_eval(
                             valueFrom[k],
@@ -727,9 +703,7 @@ class WorkflowJob:
                 runtimeContext = runtimeContext.copy()
                 runtimeContext.postScatterEval = postScatterEval
 
-                emptyscatter = [
-                    shortname(s) for s in scatter if len(cast(Sized, inputobj[s])) == 0
-                ]
+                emptyscatter = [shortname(s) for s in scatter if len(cast(Sized, inputobj[s])) == 0]
                 if emptyscatter:
                     _logger.warning(
                         "[job %s] Notice: scattering over empty input in "
@@ -739,9 +713,7 @@ class WorkflowJob:
                     )
 
                 if method == "dotproduct" or method is None:
-                    jobs = dotproduct_scatter(
-                        step, inputobj, scatter, callback, runtimeContext
-                    )
+                    jobs = dotproduct_scatter(step, inputobj, scatter, callback, runtimeContext)
                 elif method == "nested_crossproduct":
                     jobs = nested_crossproduct_scatter(
                         step, inputobj, scatter, callback, runtimeContext
@@ -752,9 +724,7 @@ class WorkflowJob:
                     )
             else:
                 if _logger.isEnabledFor(logging.DEBUG):
-                    _logger.debug(
-                        "[%s] job input %s", step.name, json_dumps(inputobj, indent=4)
-                    )
+                    _logger.debug("[%s] job input %s", step.name, json_dumps(inputobj, indent=4))
 
                 inputobj = postScatterEval(inputobj)
                 if inputobj is not None:
@@ -814,13 +784,9 @@ class WorkflowJob:
             with SourceLine(self.tool["inputs"], index, WorkflowException, debug):
                 inp_id = shortname(inp["id"])
                 if inp_id in joborder:
-                    self.state[inp["id"]] = WorkflowStateItem(
-                        inp, joborder[inp_id], "success"
-                    )
+                    self.state[inp["id"]] = WorkflowStateItem(inp, joborder[inp_id], "success")
                 elif "default" in inp:
-                    self.state[inp["id"]] = WorkflowStateItem(
-                        inp, inp["default"], "success"
-                    )
+                    self.state[inp["id"]] = WorkflowStateItem(inp, inp["default"], "success")
                 else:
                     raise WorkflowException(
                         "Input '%s' not in input object and does not have a "
@@ -844,9 +810,7 @@ class WorkflowJob:
 
                 if not step.submitted:
                     try:
-                        step.iterable = self.try_make_job(
-                            step, output_callback, runtimeContext
-                        )
+                        step.iterable = self.try_make_job(step, output_callback, runtimeContext)
                     except WorkflowException as exc:
                         _logger.error("[%s] Cannot make job: %s", step.name, str(exc))
                         _logger.debug("", exc_info=True)
@@ -886,7 +850,7 @@ class WorkflowJob:
 
 
 class WorkflowJobLoopStep:
-    """Generated for each step in Workflow.steps() containing a http://commonwl.org/cwltool#Loop requirement."""
+    """Generated for each step in Workflow.steps() containing a Loop requirement."""
 
     def __init__(self, step: WorkflowJobStep, container_engine: str):
         """Initialize this WorkflowJobLoopStep."""
@@ -905,9 +869,7 @@ class WorkflowJobLoopStep:
             if "id" in i:
                 iid = cast(str, i["id"])
                 if loop_req.get("outputMethod") == "all":
-                    self.output_buffer[iid] = cast(
-                        MutableSequence[Optional[CWLOutputType]], []
-                    )
+                    self.output_buffer[iid] = cast(MutableSequence[Optional[CWLOutputType]], [])
                 else:
                     self.output_buffer[iid] = None
 
@@ -1033,7 +995,6 @@ class WorkflowJobLoopStep:
                 self.processStatus = processStatus
 
             if processStatus not in ("success", "skipped"):
-
                 _logger.warning(
                     "[%s] Iteration %i completed %s",
                     self.step.name,
@@ -1087,9 +1048,7 @@ class WorkflowJobLoopStep:
 
             for k, v in inputobj.items():
                 if k in valueFrom:
-                    adjustDirObjs(
-                        v, functools.partial(get_listing, fs_access, recursive=True)
-                    )
+                    adjustDirObjs(v, functools.partial(get_listing, fs_access, recursive=True))
                     inputobj[k] = cast(
                         CWLObjectType,
                         expression.do_eval(
