@@ -5,6 +5,8 @@ from pathlib import Path
 from stat import S_IWGRP, S_IWOTH, S_IWRITE
 from typing import Any
 
+import pytest
+
 from cwltool.factory import Factory
 from cwltool.main import main
 
@@ -246,7 +248,9 @@ def test_iwdr_permutations_inplace(tmp_path_factory: Any) -> None:
 
 
 @needs_singularity
-def test_iwdr_permutations_singularity(tmp_path_factory: Any) -> None:
+def test_iwdr_permutations_singularity(
+    tmp_path_factory: pytest.TempPathFactory, monkeypatch: pytest.MonkeyPatch
+) -> None:
     misc = tmp_path_factory.mktemp("misc")
     fifth = misc / "fifth"
     fifth.mkdir()
@@ -269,6 +273,8 @@ def test_iwdr_permutations_singularity(tmp_path_factory: Any) -> None:
     twelfth = misc / "twelfth"
     twelfth.touch()
     outdir = str(tmp_path_factory.mktemp("outdir"))
+    singularity_dir = str(tmp_path_factory.mktemp("singularity"))
+    monkeypatch.setenv("CWL_SINGULARITY_CACHE", singularity_dir)
     err_code, stdout, _ = get_main_output(
         [
             "--outdir",
@@ -306,7 +312,9 @@ def test_iwdr_permutations_singularity(tmp_path_factory: Any) -> None:
 
 
 @needs_singularity
-def test_iwdr_permutations_singularity_inplace(tmp_path_factory: Any) -> None:
+def test_iwdr_permutations_singularity_inplace(
+    tmp_path_factory: pytest.TempPathFactory, monkeypatch: pytest.MonkeyPatch
+) -> None:
     """IWDR tests using --singularity and a forced InplaceUpdateRequirement."""
     misc = tmp_path_factory.mktemp("misc")
     fifth = misc / "fifth"
@@ -330,6 +338,8 @@ def test_iwdr_permutations_singularity_inplace(tmp_path_factory: Any) -> None:
     twelfth = misc / "twelfth"
     twelfth.touch()
     outdir = str(tmp_path_factory.mktemp("outdir"))
+    singularity_dir = str(tmp_path_factory.mktemp("singularity"))
+    monkeypatch.setenv("CWL_SINGULARITY_CACHE", singularity_dir)
     assert (
         main(
             [
