@@ -13,9 +13,9 @@ else:
 
 
 @needs_docker
-def test_for_910() -> None:
-    assert main([get_data("tests/wf/910.cwl")]) == 0
-    assert main([get_data("tests/wf/910.cwl")]) == 0
+def test_for_910(tmp_path: Path) -> None:
+    assert main(["--outdir", str(tmp_path), get_data("tests/wf/910.cwl")]) == 0
+    assert main(["--outdir", str(tmp_path), get_data("tests/wf/910.cwl")]) == 0
 
 
 @needs_docker
