@@ -109,13 +109,9 @@ def test_commandLineTool_job_tmpdir_prefix(tmp_path: Path) -> None:
 
 
 @needs_docker
-def test_dockerfile_tmpdir_prefix(
-    tmp_path: Path, monkeypatch: pytest.MonkeyPatch
-) -> None:
+def test_dockerfile_tmpdir_prefix(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
     """Test that DockerCommandLineJob.get_image respects temp directory directives."""
-    monkeypatch.setattr(
-        target=subprocess, name="check_call", value=lambda *args, **kwargs: True
-    )
+    monkeypatch.setattr(target=subprocess, name="check_call", value=lambda *args, **kwargs: True)
     (tmp_path / "out").mkdir()
     tmp_outdir_prefix = tmp_path / "out" / "1"
     (tmp_path / "3").mkdir()
@@ -141,7 +137,7 @@ def test_dockerfile_tmpdir_prefix(
         False,
         False,
         False,
-        "",
+        "no_listing",
         runtime_context.get_outdir(),
         runtime_context.get_tmpdir(),
         runtime_context.get_stagedir(),
@@ -190,7 +186,7 @@ def test_docker_tmpdir_prefix(tmp_path: Path) -> None:
         False,
         False,
         False,
-        "",
+        "no_listing",
         runtime_context.get_outdir(),
         runtime_context.get_tmpdir(),
         runtime_context.get_stagedir(),
@@ -220,9 +216,7 @@ def test_docker_tmpdir_prefix(tmp_path: Path) -> None:
         resolved=str(resolved_writable_dir), target="bar", type=None, staged=None
     )
     (tmp_path / "2").mkdir()
-    job.add_writable_directory_volume(
-        runtime, volume_dir, None, str(tmp_path / "2" / "dir")
-    )
+    job.add_writable_directory_volume(runtime, volume_dir, None, str(tmp_path / "2" / "dir"))
     children = sorted((tmp_path / "2").glob("*"))
     assert len(children) == 1
     subdir = tmp_path / "2" / children[0]
@@ -235,9 +229,7 @@ def test_docker_tmpdir_prefix(tmp_path: Path) -> None:
 
     volume_file = MapperEnt(resolved="Hoopla!", target="baz", type=None, staged=None)
     (tmp_path / "4").mkdir()
-    job.create_file_and_add_volume(
-        runtime, volume_file, None, None, str(tmp_path / "4" / "file")
-    )
+    job.create_file_and_add_volume(runtime, volume_file, None, None, str(tmp_path / "4" / "file"))
     children = sorted((tmp_path / "4").glob("*"))
     assert len(children) == 1
     subdir = tmp_path / "4" / children[0]
